
function [bestX,bestFx]=AOAfx(fx,Lb,Ub,MaxPop,MaxIters)

  nVars = length(Lb);
  bestX = Lb + (Ub-Lb)/2;
  bestFx = fx(bestX);
  lastIter = 0;
  MaxStall = fix(MaxIters/20);
  
  %Initialize the positions of solution
  X = zeros(MaxPop, nVars);
  for i=1:MaxPop
    X(i,:) = Lb + (Ub - Lb).*rand(1,nVars);
  end
  Xnew=X;
  Ffun=zeros(MaxPop);% (fitness values)
  Ffun_new=zeros(MaxPop);% (fitness values)
  
  MOP_Max=1;
  MOP_Min=0.2;
  Alpha=5;
  Mu=0.499;
  
  for i=1:MaxPop
    Ffun(i)=fx(X(i,:));  %Calculate the fitness values of solutions
    if Ffun(i) < bestFx
      bestFx=Ffun(i);
      bestX=X(i,:);
    end
  end
  
  for iter=1:MaxIters  %Main loop
  
    if (iter - lastter) > MaxStall
      lastIter = iter;
      for i=1:MaxPop
        X(i,:) = Lb + (Ub - Lb).*rand(1,nVars);
        Ffun(i)=fx(X(i,:));
        if Ffun(i) < bestFx
          bestFx=Ffun(i);
          bestX=X(i,:);
        end
      end
    end
  
    MOP=1-((iter)^(1/Alpha)/(MaxIters)^(1/Alpha));   % Probability Ratio
    MOA=MOP_Min+iter*((MOP_Max-MOP_Min)/MaxIters); %Accelerated function
  
    %Update the Position of solutions
    for i=1:MaxPop   % if each of the Ub and Lb has a just value
      for j=1:nVars
        r1=rand();
        if r1 < MOA
          r2=rand();
          if r2 > 0.5
            Xnew(i,j)=bestX(1,j)/(MOP+eps)*((Ub(j)-Lb(j))*Mu+Lb(j));
          else
            Xnew(i,j)=bestX(1,j)*MOP*((Ub(j)-Lb(j))*Mu+Lb(j));
          end
        else
          r3=rand();
          if r3 > 0.5
            Xnew(i,j)=bestX(1,j)-MOP*((Ub(j)-Lb(j))*Mu+Lb(j));
          else
            Xnew(i,j)=bestX(1,j)+MOP*((Ub(j)-Lb(j))*Mu+Lb(j));
          end
        end
      end
  
      for j=1:nVars
        if Xnew(i,j) < Lb(j) || Xnew(i,j) > Ub(j)
          Xnew(i,j) = Lb(j) + (Ub(j) - Lb(j))*rand;
        end
      end
  
      Ffun_new(i)=fx(Xnew(i,:));  % calculate Fitness function
      if Ffun_new(i) < Ffun(i)
        X(i,:)=Xnew(i,:);
        Ffun(i)=Ffun_new(i);
      end
  
      if Ffun(i) < bestFx
        lastIter = iter;
        bestFx=Ffun(i);
        bestX=X(i,:);
        fprintf("Iters: %d, Fx = %e, X=[", iter, bestFx);
        fprintf("%f, ", bestX(1:nVars-1));
        fprintf("%f]\n", bestX(nVars));
      end
    end
  end
end



