
% Developed in MATLAB R2017b
% Source codes demo version 1.0
% _____________________________________________________
%  Author, inventor and programmer: Iraj Naruei and Farshid Keynia,
%  e-Mail: irajnaruei@iauk.ac.ir , irajnaruei@yahoo.com
% _____________________________________________________
%  Co-author and Advisor: Farshid Keynia
%
%         e-Mail: fkeynia@gmail.com
% _____________________________________________________
%  Co-authors: Amir Sabbagh Molahoseyni
%        
%         e-Mail: sabbagh@iauk.ac.ir
% _____________________________________________________
% You can find the HPO article at 
% Hunter?rey optimization: algorithm and applications
% https://doi.org/10.1007/s00500-021-06401-0
% _____________________________________________________
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Very good function!!

function [bestX,bestFx] = HPO(fx,Lb,Ub,MaxPop,MaxIters,B)
  % Constriction Coeficient
  if nargin < 6, B = 0.1; end
  nVars = length(Lb);

  %% Initialization
  HPpos=rand(MaxPop,nVars).*(Ub-Lb)+Lb;
  HPposFitness = zeros(MaxPop,1);
  % for i=1:MaxPop
  %     HPposFitness(i)=inf;
  % end
  % Evaluate
  for i=1:MaxPop
    HPposFitness(i)=fx(HPpos(i,:));
  end
  % NFE = MaxPop;
  [~,indx] = min(HPposFitness);
  %
  bestX = HPpos(indx,:);   % bestX HPO
  bestFx =HPposFitness(indx);
  lastIter = 0;
  MaxStall = fix(MaxIters/20);
  %% HPO Main Loop
  for iter = 1:MaxIters
    if (iter - lastIter) >= MaxStall
      lastIter = iter;
      fprintf("---------------------------- New population at iter %d\n", iter);
      HPpos=rand(MaxPop,nVars).*(Ub-Lb)+Lb;
      for i=1:MaxPop
        HPposFitness(i)=fx(HPpos(i,:));
      end
     [~,indx] = min(HPposFitness);
     %
     bestX2 = HPpos(indx,:);   % bestX HPO
     bestFx2 =HPposFitness(indx);
     if bestFx2 < bestFx
       bestFx = bestFx2;
       bestX = bestX2;
     end
    end
    c = 1 - (iter-1)*((0.98)/MaxIters);   % Update C Parameter
    kbest=round(MaxPop*c);        % Update kbest
    for i = 1:MaxPop
      r1=rand(1,nVars) < c;
      r2=rand;
      r3=rand(1,nVars);
      idx=(r1==0);
      z=r2.*idx+r3.*~idx;
      if rand < B
        xi=mean(HPpos);
        dist = pdist2(xi,HPpos);
        [~,idxsortdist]=sort(dist);
        SI=HPpos(idxsortdist(kbest),:);
        HPpos(i,:) =HPpos(i,:)+0.5*((2*(c)*z.*SI-HPpos(i,:))+(2*(1-c)*z.*xi-HPpos(i,:)));
      else
        for j=1:nVars
          rr=-1+2*z(j);
          HPpos(i,j)= 2*z(j)*cos(2*pi*rr)*(bestX(j)-HPpos(i,j))+bestX(j);
        end
      end
      % HPpos(i,:) = min(max(HPpos(i,:),Lb),Ub);
      for j=1:nVars
        if HPpos(i,j) < Lb(j) || HPpos(i,j) > Ub(j)
          HPpos(i,j) = Lb(j) + (Ub(j) - Lb(j))*rand;
        end
      end
      % Evaluation
      HPposFitness(i) = fx(HPpos(i,:));
      % Update bestX
      if HPposFitness(i) < bestFx
        lastIter = iter;
        bestX = HPpos(i,:);
        bestFx = HPposFitness(i);
        fprintf("Iter: %d, Fx = %e, X=[", iter, bestFx);
        fprintf("%f, ", bestX(1:nVars-1));
        fprintf("%f]\n", bestX(nVars));
      end
    end
  
  end
end
