clc
close all
clear all

global mdl
global xdata
global ydata


warning("off")

dt = datetime('now','TimeZone','local','Format','y-MMM-d HH_mm_ss');
dtstr = string(dt);
filename = strcat(pwd,'\bestrathet2.xlsx');
outFile = strcat("bestrathet2_",dtstr,".txt");
diary(outFile)

fprintf("Program bestrathet2\n\n");
fprintf("%s\n", dtstr);
fprintf("Please wait ...\n\n");

fprintf("Excel file used is %s\n", filename);
fprintf("Diary file used is %s\n", outFile);

xyData = readmatrix(filename,'Sheet','Data');
[maxrows,~] = size(xyData);
ydata = xyData(:,1);
xdata = xyData(:,2);
xyTransf = readmatrix(filename,'Sheet','Transf');
yPwrs = xyTransf(1:2,1);
x1Pwrs = xyTransf(1:2,2);
x2Pwrs = xyTransf(1:2,3);

MaxIters = 1000;
MaxPop= 200;
itersToReset = 50;
Lb = [yPwrs(1) x1Pwrs(1) x2Pwrs(1)];
Ub = [yPwrs(2) x1Pwrs(2) x2Pwrs(2)];

[gBestX,gBestCost] = pso(@myFit,Lb,Ub,MaxPop,MaxIters,itersToReset)
r = myFit(gBestX);
fprintf("Adjusted R-square = ");
format long
AdjR2 = 1 - r

fprintf("Regression model\n")
mdl
res = zeros(2,3);
res(1,1) = gBestX(1);
res(1,2) = gBestX(2);
res(1,3) = gBestX(3);
res(2,1) = mdl.Coefficients{1,1};
res(2,2) = mdl.Coefficients{2,1};
res(2,3) = mdl.Coefficients{3,1};
fprintf("\nResults (powers/coefficients\n")
res % display res as is
writematrix(res(1,1), filename, 'Sheet', 'Results', 'Range', 'A2:A3')
writematrix(res(1,2:3), filename, 'Sheet', 'Results', 'Range', 'C2:D3');
writematrix(res(2,:), filename, 'Sheet', 'Results', 'Range', 'A3:C3');

% now dow projections
n = length(ydata);
ycalc = (res(2,1) + res(2,2).*xdata.^gBestX(2))./(1+res(2,3).*xdata.^gBestX(3));
ycalc = ycalc.^(1/gBestX(1));
percErr = 100.*(ycalc - ydata)./ydata;
ProjMat = [ydata xdata ycalc percErr];
writematrix(ProjMat, filename, 'Sheet', 'Project', 'Range', strcat("A2:D", num2str(n)));

format short
fprintf("Done!\n");
diary off

function r = myFit(coeff)
  global mdl
  global xdata
  global ydata
  
  y = ydata.^coeff(1);
  x1 = xdata(:,1).^coeff(2);
  x2 = xdata(:,1).^coeff(3);
  x2 = -1.*y.*x2;
  X = [x1 x2];
  mdl = fitlm(X,y);
  r = 1 - mdl.Rsquared.Adjusted;
end

diary off