clc
close all
clear all

global mdl
global xdata
global ydata
global xVarIdx

warning("off")
dt = datetime('now','TimeZone','local','Format','y-MMM-d HH_mm_ss');
dtstr = string(dt);
filename = strcat(pwd,'\bestrathetFlex1b.xlsx');
outFile = strcat("bestrathetFlex1b_",dtstr,".txt");
diary(outFile)

fprintf("Program bestrathetFlex1b\n\n");
fprintf("%s\n", dtstr);
fprintf("Please wait ...\n\n");
fprintf("Excel file used is %s\n", filename);
fprintf("Diary file used is %s\n", outFile);
xyData = readmatrix(filename,'Sheet','Data');
[maxrows,~] = size(xyData);
ydata = xyData(:,1);
xdata = xyData(:,2:end);
xyTransf = readmatrix(filename,'Sheet','Transf');
[~,nVars] = size(xyTransf);
nXvars = nVars - 1;
xVarIdx = xyTransf(1,2:end);
Lb = xyTransf(2,:);
Ub = xyTransf(3,:);

MaxIters = 2000;
MaxPop= 250;
itersToReset = 75;
[gBestX,gBestCost] = psox9(@myFit,Lb,Ub,MaxPop,MaxIters,itersToReset)
fprintf("Regression model\n")
r = myFit(gBestX);
fprintf("Adjusted R-square = ");
format long
AdjR2 = 1 - r
writematrix(AdjR2, filename, 'Sheet', 'R2', 'Range', 'A2:A2');
mdl
res = zeros(2,nVars);
res(1,:) = gBestX;
res(2,:) = mdl.Coefficients{:,1};
writematrix(res(1,1), filename, 'Sheet', 'Results', 'Range', 'A2:A2');
writematrix(res(1,2:end), filename, 'Sheet', 'Results', 'Range', 'C2:Z2');
writematrix(res(2,:), filename, 'Sheet', 'Results', 'Range', 'B3:Z3');
fprintf("\nResults (powers/coefficients)\n")
res % display res as is

% now do the projections
ycalc = project(res);
percErr = 100.*(ycalc-ydata)./ydata;
ProjMat = [ydata xdata ycalc percErr];
writematrix(ProjMat, filename, 'Sheet', 'Project', 'Range', strcat("A2:Z", num2str(length(ydata))));

% ----------------------------- Phase 2 with yp = 1


fprintf("\n\n----------------- Phase 2 with yp = 1 -----------------\n\n")
Lb(1) = 1;
Ub(1) = 1;

[gBestX,gBestCost] = psox9(@myFit,Lb,Ub,MaxPop,MaxIters,itersToReset)
fprintf("Regression model\n")

r = myFit(gBestX);
fprintf("Adjusted R-square = ");
format long
AdjR2 = 1 - r
writematrix(AdjR2, filename, 'Sheet', 'R2', 'Range', 'B2:B2');
mdl
res2 = zeros(2,nVars);
res2(1,:) = gBestX;
res2(2,:) = mdl.Coefficients{:,1};
writematrix(res2(1,1), filename, 'Sheet', 'Results2', 'Range', 'A2:A2');
writematrix(res2(1,2:end), filename, 'Sheet', 'Results2', 'Range', 'C2:Z2');
writematrix(res2(2,:), filename, 'Sheet', 'Results2', 'Range', 'B3:Z3');
fprintf("\nResults (powers/coefficients)\n")
res2 % display res as is

% now do the projections
ycalc2 = project(res2);
percErr2 = 100.*(ycalc2-ydata)./ydata;
ProjMat = [ydata xdata ycalc2 percErr2];
writematrix(ProjMat, filename, 'Sheet', 'Project2', 'Range', strcat("A2:Z", num2str(length(ydata))));

format short
fprintf("Done!\n");
diary off
beep on
for i=1:3
  beep;
  pause(1);
end

function r = myFit(coeff)
  global mdl
  global xdata
  global ydata
  global xVarIdx

  y = ydata.^coeff(1);
  X = [];
  for i = 1:length(xVarIdx)
    j = abs(xVarIdx(i));
    x = xdata(:,j).^coeff(i+1);
    if xVarIdx(i) < 0
      x = -1.*y.*x;
    end
    X = [X x];
  end
  mdl = fitlm(X,y);
  r = 1 - mdl.Rsquared.Adjusted;
end

function ycalc = project(res)
  global xdata
  global ydata
  global xVarIdx

  n = length(ydata);
  yPwrInv = 1/res(1,1);
  totTerms = length(xVarIdx);
  ycalc = zeros(n,1);
  for i=1:n
    try
      sum1 = res(2,1);
      sum2 = 1;
      for j=1:totTerms
        if xVarIdx(j) >=0 
          k = xVarIdx(j);
          sum1 = sum1 + res(2,j+1)*xdata(i,k)^res(1,j+1);
        else
          k = abs(xVarIdx(j));
          sum2 = sum2 + res(2,j+1)*xdata(i,k)^res(1,j+1);
        end
      end
      ycalc(i) = (sum1 / sum2)^yPwrInv;
    catch ME
      ycalc(i) = Inf;
    end
  end

end