function [bestMdl, bestPwrs] = ...
  bestrathetFlexHill(XlFile, outFile, MaxIters)

  global mdl
  global xdata
  global ydata
  global xVarIdx
  
  warning("off")
  
  diary(outFile)

  if nargin < 3, MaxIters = 1000000; end

  MaxIters2 = 100;
  MaxIters = fix(MaxIters/MaxIters2/2);
  StallItersToStop = fix(MaxIters/10);
  
  
  dt = datetime('now','TimeZone','local','Format','y-MMM-d HH_mm_ss');
  dtstr = string(dt);
  
  fprintf("Please wait ...\n\n");
  fprintf("%s\n", dtstr);
  fprintf("Function bestrathetFlexHill\n\n");
  fprintf("Excel file used is %s\n", XlFile);
  fprintf("Diary file used is %s\n", outFile);
  xyData = readmatrix(XlFile,'Sheet','Data');
  ydata = xyData(:,1);
  xdata = xyData(:,2:end);
  xyTransf = readmatrix(XlFile,'Sheet','Transf');
  [~,nVars] = size(xyTransf);
  nXvars = nVars - 1;
  xVarIdx = xyTransf(1,2:end);
  Lb = xyTransf(2,:);
  Ub = xyTransf(3,:);
  n = length(Lb);
 
  lastIter = 0;
  gBestX = Lb + (Ub - Lb).*rand(1,nVars);
  dummy= myFit(gBestX);
  bestSSE = mdl.SSE;
  for iter=1:MaxIters
    pop = gBestX;
    % change one variable at a time
    for i=1:n
      for iter2=1:MaxIters2
        pop(i) = Lb(i) + (Ub(i) - Lb(i)).*rand;
        dummy= myFit(pop);
        if mdl.SSE < bestSSE
          gBestX = pop;
          bestSSE = mdl.SSE;
          fprintf("Iter: %d/%d/%d, ", iter, iter2, i);
          fprintf("Best Fx =%e, X=[", bestSSE);
          fprintf("%f, ", gBestX(1:nVars-1));
          fprintf("%f]\n", gBestX(nVars));
          lastIter = iter;
        end      
        if (iter - lastIter) >= StallItersToStop, break; end
      end
      end
    if (iter - lastIter) < StallItersToStop
      for iter2=1:MaxIters2
        % now try change all the variables
        pop = Lb + (Ub - Lb).*rand(1,nVars);
        dummy= myFit(pop);
        if mdl.SSE < bestSSE
          gBestX = pop;
          bestSSE = mdl.SSE;
          fprintf("Iter: %d/%d, ", iter, iter2);
          fprintf("Best Fx =%e, X=[", bestSSE);
          fprintf("%f, ", gBestX(1:nVars-1));
          fprintf("%f]\n", gBestX(nVars));
          lastIter = iter;
        end      
        if (iter - lastIter) >= StallItersToStop, break; end
      end
    end
  end
  fprintf("\nResults (powers/coefficients)\n")
  bestPwrs = gBestX
  fprintf("Regression model\n")
  dummy = myFit(gBestX);
  format long
  AdjR2 = mdl.Rsquared.Adjusted;
  fprintf("Adj R-sqr = %14.8f\n", AdjR2);
  writematrix(AdjR2, XlFile, 'Sheet', 'R2', 'Range', 'A2:A2');
  bestMdl = mdl 

  res = zeros(2,nVars+1);
  res(1,1)= gBestX(1);
  res(1,3:end) = gBestX(2:end);
  res(2,2:end) = mdl.Coefficients{:,1};
  writematrix(res(1,1), XlFile, 'Sheet', 'Results', 'Range', 'A2:A2');
  writematrix(res(1,3:end), XlFile, 'Sheet', 'Results', 'Range', 'C2:Z2');
  writematrix(res(2,2:end), XlFile, 'Sheet', 'Results', 'Range', 'B3:Z3');
  
  % now do the projections
  ycalc = project(res);
  percErr = 100.*(ycalc-ydata)./ydata;
  ProjMat = [ydata xdata ycalc percErr];
  writematrix(ProjMat, XlFile, 'Sheet', 'Project', 'Range', strcat("A2:Z", num2str(length(ydata))));
  
  format short
  diary off

  beepit(4)
end

function beepit(nTimes)
  for i=1:nTimes
    beep;
    pause(1);
  end
end

function r = myFit(coeff)
  global mdl
  global xdata
  global ydata
  global xVarIdx

  if coeff(1) >= 0
    y = ydata.^coeff(1);
  else
    y = 1./ydata.^abs(coeff(1));
  end
  X = [];
  for i = 1:length(xVarIdx)
    j = abs(xVarIdx(i));
    if coeff(i+1) >= 0
      x = xdata(:,j).^coeff(i+1);
    else 
      x = 1./xdata(:,j).^abs(coeff(i+1));
    end
    if xVarIdx(i) < 0
      x = -1.*y.*x;
    end
    X = [X x];
  end
  mdl = fitlm(X,y,'RobustOpts','on');
  r = mdl.SSE;
  if isnan(r)
    r = 1e99;
  end
end

function ycalc = project(res)
  global xdata
  global ydata
  global xVarIdx

  n = length(ydata);
  yPwrInv = 1/res(1,1);
  totTerms = length(xVarIdx);
  ycalc = zeros(n,1);
  for i=1:n
    try
      sum1 = res(2,2);
      sum2 = 1;
      for j=1:totTerms
        p = j+2;
        if xVarIdx(j) >=0 
          k = xVarIdx(j);
          sum1 = sum1 + res(2,p)*xdata(i,k)^res(1,p);
        else
          k = abs(xVarIdx(j));
          sum2 = sum2 + res(2,p)*xdata(i,k)^res(1,p);
        end
      end
      ycalc(i) = (sum1 / sum2)^yPwrInv;
    catch ME
      ycalc(i) = Inf;
    end
  end

end