function [mdl1, mdl2, bestPwrs1, bestPwrs2] = ...
  bestrathetFlexMatlabPSO(XlFile, outFile, MaxIters, MaxPop, MaxStallIterations)

  global mdl
  global xdata
  global ydata
  global xVarIdx
  
  warning("off")

  diary(outFile)

  if nargin < 5, MaxStallIterations = 75; end
  if nargin < 4, MaxPop = 500; end
  if nargin < 3, MaxIters = 5000; end
  
  dt = datetime('now','TimeZone','local','Format','y-MMM-d HH_mm_ss');
  dtstr = string(dt);

  fprintf("Please wait ...\n\n");
  fprintf("%s\n", dtstr);
  fprintf("Program bestrathetFlexMatlabPSO1\n\n");
  fprintf("Excel file used is %s\n", XlFile);
  fprintf("Diary file used is %s\n", outFile);
  xyData = readmatrix(XlFile,'Sheet','Data');
  [maxrows,nVars] = size(xyData);
  ydata = xyData(:,1);
  xdata = xyData(:,2:end);
  xyTransf = readmatrix(XlFile,'Sheet','Transf');
  [~,nVars] = size(xyTransf);
  nXvars = nVars - 1;
  xVarIdx = xyTransf(1,2:end);
  Lb = xyTransf(2,:);
  Ub = xyTransf(3,:);
  
  options = optimoptions('particleswarm','MaxIterations',MaxIters,'SwarmSize',MaxPop, ...
    'MaxTime',30*60,'MaxStallIterations',MaxStallIterations, ...
    'Display', 'iter','FunctionTolerance',1e-8);
  fprintf("\nResults (powers/coefficients)\n")
  gBestX = particleswarm(@myFit,length(Lb),Lb,Ub,options)
  bestPwrs1= gBestX;
  bestPwrs2= gBestX;
  fprintf("Regression model\n")
  r = myFit(gBestX);
  format long
  AdjR2 = 1 - r;
  fprintf("Adj R-sqr = %14.8f\n", AdjR2);
  writematrix(AdjR2, XlFile, 'Sheet', 'R2', 'Range', 'A2:A2');
  mdl1=mdl
  mdl2 = mdl;

  res = zeros(2,nVars+1);
  res(1,1)= gBestX(1);
  res(1,3:end) = gBestX(2:end);
  res(2,2:end) = mdl.Coefficients{:,1};
  writematrix(res(1,1), XlFile, 'Sheet', 'Results', 'Range', 'A2:A2');
  writematrix(res(1,3:end), XlFile, 'Sheet', 'Results', 'Range', 'C2:Z2');
  writematrix(res(2,2:end), XlFile, 'Sheet', 'Results', 'Range', 'B3:Z3');

  
  % now do the projections
  [ycalc,ycalcT] = project(res);
  percErr = 100.*(ycalc-ydata)./ydata;
  AdjustedR2 = rsqr(ydata.^res(1,1),ycalcT,length(xVarIdx));
  fprintf("Adj R-sqr (transformed y) = %14.8f\n", AdjustedR2);  
  AdjustedR3 = rsqr(ydata,ycalc,length(xVarIdx));
  fprintf("Adj R-sqr (untransformed y) = %14.8f\n", AdjustedR3);
  ProjMat = [ydata xdata ycalc percErr];
  writematrix(ProjMat, XlFile, 'Sheet', 'Project', 'Range', strcat("A2:Z", num2str(length(ydata))));
  

  if gBestX(1) ~= 1

    fprintf("\n\n----------------- Phase 2 with yp = 1 -----------------\n\n")
    Lb(1) = 1;
    Ub(1) = 1;
  
    fprintf("\nResults (powers/coefficients)\n")  
    gBestX = particleswarm(@myFit,length(Lb),Lb,Ub,options)
    bestPwrs2= gBestX;
    fprintf("Regression model\n")
    
    r = myFit(gBestX);
    format long
    AdjR2 = 1 - r;
    fprintf("Adj R-sqr = %14.8f\n", AdjR2)
   
    writematrix(AdjR2, XlFile, 'Sheet', 'R2', 'Range', 'B2:B2');
    mdl2=mdl
   
    res2 = zeros(2,nVars+1);
    res2(1,1)= gBestX(1);
    res2(1,3:end) = gBestX(2:end);
    res2(2,2:end) = mdl.Coefficients{:,1};
    writematrix(res2(1,1), XlFile, 'Sheet', 'Results2', 'Range', 'A2:A2');
    writematrix(res2(1,3:end), XlFile, 'Sheet', 'Results2', 'Range', 'C2:Z2');
    writematrix(res2(2,2:end), XlFile, 'Sheet', 'Results2', 'Range', 'B3:Z3');
  
    
   % now do the projections
  [ycalc,ycalcT] = project(res);
  percErr2 = 100.*(ycalc-ydata)./ydata;
  AdjustedR2 = rsqr(ydata.^res(1,1),ycalcT,length(xVarIdx));
  fprintf("Adj R-sqr (transformed y) = %14.8f\n", AdjustedR2);  
  AdjustedR3 = rsqr(ydata,ycalc,length(xVarIdx));
  fprintf("Adj R-sqr (untransformed y) = %14.8f\n", AdjustedR3);
  ProjMat2 = [ydata xdata ycalc2 percErr2];
    writematrix(ProjMat2, XlFile, 'Sheet', 'Project2', 'Range', strcat("A2:Z", num2str(length(ydata))));
  end  

  format short
  diary off
  beep on
  for i=1:3
    beep;
    pause(1);
  end
end

function r = myFit(coeff)
  global mdl
  global xdata
  global ydata
  global xVarIdx

  if coeff(1) >= 0
    y = ydata.^coeff(1);
  else
    y = 1./ydata.^abs(coeff(1));
  end
  X = [];
  for i = 1:length(xVarIdx)
    j = abs(xVarIdx(i));
    if coeff(i+1) >= 0
      x = xdata(:,j).^coeff(i+1);
    else 
      x = 1./xdata(:,j).^abs(coeff(i+1));
    end
    if xVarIdx(i) < 0
      x = -1.*y.*x;
    end
    X = [X x];
  end
  mdl = fitlm(X,y,'RobustOpts','on');
  r = 1 - mdl.Rsquared.Adjusted;
  if isnan(r)
    r = 1;
  end
end

function [ycalc,ycalcT] = project(res)
  global xdata
  global ydata
  global xVarIdx

  n = length(ydata);
  yPwrInv = 1/res(1,1);
  totTerms = length(xVarIdx);
  ycalc = zeros(n,1);
  ycalcT = zeros(n,1);
  for i=1:n
    try
      sum1 = res(2,2);
      sum2 = 1;
      for j=1:totTerms
        k = abs(xVarIdx(j));
        if xVarIdx(j) >=0 
          if res(1,j+2) >= 0
            z = res(2,j+2)*xdata(i,k)^res(1,j+2);
          else
            z = res(2,j+2)/xdata(i,k)^abs(res(1,j+2));
          end
           sum1 = sum1 + z;
        else
          if res(1,j+2) >= 0
            z = res(2,j+2)*xdata(i,k)^res(1,j+2);
          else 
            z = res(2,j+2)/xdata(i,k)^abs(res(1,j+2));
          end
          sum2 = sum2 + z;
        end
      end
      ycalcT(i) = (sum1 / sum2);
      if yPwrInv >= 0
        ycalc(i) = ycalcT(i)^yPwrInv;
      else
        ycalc(i) = ycalcT(i)^abs(yPwrInv);
      end
    catch ME
      ycalc(i) = Inf;
      yclcT(i) = inf;
    end
  end

end