% The EVPS code in MATLAB
% Enhanced VIBRATING PARTICLES SYSTEM - EVPS
function [bestX, bestFx] = evpsFx(fx,Lb,Ub,popSize,maxIters)

% Example
% [bestX, bestFx] = evpsFx(@fx1,[0 0 0 0],5*[5 5 5 5],50,500)
  % Initializing variables
  MaxStall = fix(maxIters/20);
  nVar = length(Lb); % Number of optimization variables
  alpha=0.05;
  w1=0.3;
  w2=0.3;
  w3=1-w1-w2;
  p=0.2;  %  With  the  probability  of  (1-p)  the  effect  of  BP
  % is ignored in updating
  PAR=0.1;
  HMCR=0.95;
  Memorysize=4;
  result=zeros(popSize,nVar+1);
  Memory=zeros(Memorysize,nVar+1);
  neighbor=0.1;  %  Parameters  for  handling  the  side constraints
  % Initializing particles
  position = zeros(popSize,nVar);
  for i=1:popSize
    position(i,:) = Lb +rand(1,nVar).*(Ub-Lb);
  end
  % Search
  agentCost=zeros(popSize,3); % Array of agent costs
  HBV=zeros(popSize,nVar+2); % Historically best matrix
  fxArr = zeros(popSize,1);
  bestFx = 1e99;
  lastIter = 0;
  for iter=1:maxIters
    if (iter-lastIter) >= MaxStall
      fprintf("------------------ New population at iter %d\n", iter);
      lastIter = iter;
      for i=1:popSize
        position(i,:) = Lb +rand(1,nVar).*(Ub-Lb);
        fxArr(i) = fx(position(i,:));
      end  
      [~,indx] = min(fxArr);
      bestFx2 = fxArr(indx);
      if bestFx2 < bestFx
        bestFx = bestFx2;
        bestX = position(indx,:);
      end
    end

    % Evaluating and storing
    for m=1:popSize
      % Evaluating the objective function for each particle
      penalizedWeight=fx(position(m,:));
      agentCost(m,1)=penalizedWeight;
      agentCost(m,2)=m;
      agentCost(m,3)=penalizedWeight; % was weight;
    end
    sortedAgentCost=sortrows(agentCost);
    for m=1:popSize
      if iter==1 || agentCost(m,1)<HBV(m,1)
        HBV(m,1)=agentCost(m,1);
        HBV(m,2)=agentCost(m,3);
        for n=1:nVar
          HBV(m,n+2)=position(m,n);
        end
      end
    end
    sortedHBV=sortrows(HBV);
    result(iter,1)=sortedHBV(1,1);
    result(iter,2:nVar+1)=sortedHBV(1,3:nVar+2);
    if iter==1
      for i=1:Memorysize
        Memory(i,1)=sortedHBV(i,1);
        Memory(i,2:nVar+1)=sortedHBV(i,3:nVar+2);
      end
    end
    if Memory(Memorysize,1)>sortedHBV(1,1)
      Memory(Memorysize,1)=sortedHBV(1,1);
      Memory(Memorysize,2:nVar+1)=sortedHBV(1,3:nVar+2);
    end
    Memory=sortrows(Memory);
    % Updating particle positions
    D=(iter/maxIters)^(-alpha);
    for m=1:popSize
      temp1=ceil(unifrnd(1,popSize/2,1));
      temp2=ceil(unifrnd(popSize/2,popSize,1));
      temp11=round(unifrnd(1,Memorysize,1));
      if p<rand
        w3=0;
        w2=1-w1;
      end
  
      for n=1:nVar
        if rand<w3
          A(m,n)=(-1)^(ceil(randn()))*(position(sortedAgentCost(temp2,2),n)-position(m,n));
          position(m,n)=D*A(m,n)*rand+position(sortedAgentCost(temp2,2),n);
        elseif rand<w2
          A(m,n)=(-1)^(ceil(randn()))*(position(sortedAgentCost(temp1,2),n)-position(m,n));
          position(m,n)=D*A(m,n)*rand+position(sortedAgentCost(temp1,2),n);
        else
          A(m,n)=(-1)^(ceil(randn()))*(Memory(temp11,1+n)-position(m,n));
          position(m,n)=D*A(m,n)*rand+Memory(temp11,1+n);
        end
      end
      w2=0.3;w3=1-w1-w2;
    end
    % Handling the side constraints
    for m=1:popSize
      for n=1:nVar
        if position(m,n)<Lb(n) || position(m,n)>Ub(n)
          temp1=rand;temp2=rand;temp3=ceil(rand*popSize);
          if temp1<=HMCR && temp2<=(1-PAR)
            position(m,n)=sortedHBV(temp3,2+n);
          elseif temp1<=HMCR && temp2>(1-PAR)
            position(m,n)=sortedHBV(temp3,2+n)+neighbor;
            if position(m,n)>Ub(n)
              position(m,n)=sortedHBV(temp3,2+n)-2*neighbor;
            end
          else
            position(m,n)=Lb(n)+(rand*(Ub(n)-Lb(n)));
          end
        end
      end
    end
    if bestFx > sortedHBV(1,1)
      bestFx = sortedHBV(1,1);
      bestX = sortedHBV(1,3:end); 
      lastIter = iter;
      fprintf("Iter: %d, Fx = %e, X=[", iter, bestFx)
      fprintf("%f, ", bestX(1:nVar-1));
      fprintf("%f]\n", bestX(nVar));
    end
  end

end